@@ -10,24 +10,29 @@ use std::net::{IpAddr, SocketAddr};
 use tokio::sync::{mpsc, Mutex};
 
 use async_trait::async_trait;
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
 
 const MAX_READ_QUEUE_SIZE: usize = 1024;
 
-// vNet implements this
+/// vNet implements this
 #[async_trait]
 pub(crate) trait ConnObserver {
     async fn write(&self, c: Box<dyn Chunk + Send + Sync>) -> Result<()>;
+    async fn on_closed(&self, addr: SocketAddr);
     fn determine_source_ip(&self, loc_ip: IpAddr, dst_ip: IpAddr) -> Option<IpAddr>;
 }
 
-// UDPConn is the implementation of the Conn and PacketConn interfaces for UDP network connections.
-// comatible with net.PacketConn and net.Conn
+pub(crate) type ChunkChTx = mpsc::Sender<Box<dyn Chunk + Send + Sync>>;
+
+/// UDPConn is the implementation of the Conn and PacketConn interfaces for UDP network connections.
+/// comatible with net.PacketConn and net.Conn
 pub(crate) struct UdpConn {
     loc_addr: SocketAddr,
     rem_addr: Mutex<Option<SocketAddr>>,
-    read_ch_tx: Arc<mpsc::Sender<Box<dyn Chunk + Send + Sync>>>,
+    read_ch_tx: Arc<Mutex<Option<ChunkChTx>>>,
     read_ch_rx: Mutex<mpsc::Receiver<Box<dyn Chunk + Send + Sync>>>,
+    closed: AtomicBool,
     obs: Arc<Mutex<dyn ConnObserver + Send + Sync>>,
 }
 
@@ -42,13 +47,14 @@ impl UdpConn {
         UdpConn {
             loc_addr,
             rem_addr: Mutex::new(rem_addr),
-            read_ch_tx: Arc::new(read_ch_tx),
+            read_ch_tx: Arc::new(Mutex::new(Some(read_ch_tx))),
             read_ch_rx: Mutex::new(read_ch_rx),
+            closed: AtomicBool::new(false),
             obs,
         }
     }
 
-    pub(crate) fn get_inbound_ch(&self) -> Arc<mpsc::Sender<Box<dyn Chunk + Send + Sync>>> {
+    pub(crate) fn get_inbound_ch(&self) -> Arc<Mutex<Option<ChunkChTx>>> {
         Arc::clone(&self.read_ch_tx)
     }
 }
@@ -66,13 +72,13 @@ impl Conn for UdpConn {
         Ok(n)
     }
 
-    // recv_from reads a packet from the connection,
-    // copying the payload into p. It returns the number of
-    // bytes copied into p and the return address that
-    // was on the packet.
-    // It returns the number of bytes read (0 <= n <= len(p))
-    // and any error encountered. Callers should always process
-    // the n > 0 bytes returned before considering the error err.
+    /// recv_from reads a packet from the connection,
+    /// copying the payload into p. It returns the number of
+    /// bytes copied into p and the return address that
+    /// was on the packet.
+    /// It returns the number of bytes read (0 <= n <= len(p))
+    /// and any error encountered. Callers should always process
+    /// the n > 0 bytes returned before considering the error err.
     async fn recv_from(&self, buf: &mut [u8]) -> Result<(usize, SocketAddr)> {
         let mut read_ch = self.read_ch_rx.lock().await;
         while let Some(chunk) = read_ch.recv().await {
@@ -106,8 +112,8 @@ impl Conn for UdpConn {
         }
     }
 
-    // send_to writes a packet with payload p to addr.
-    // send_to can be made to time out and return
+    /// send_to writes a packet with payload p to addr.
+    /// send_to can be made to time out and return
     async fn send_to(&self, buf: &[u8], target: SocketAddr) -> Result<usize> {
         let src_ip = {
             let obs = self.obs.lock().await;
@@ -133,4 +139,21 @@ impl Conn for UdpConn {
     async fn local_addr(&self) -> Result<SocketAddr> {
         Ok(self.loc_addr)
     }
+
+    async fn close(&self) -> Result<()> {
+        if self.closed.load(Ordering::SeqCst) {
+            return Err(Error::ErrAlreadyClosed.into());
+        }
+        self.closed.store(true, Ordering::SeqCst);
+        {
+            let mut reach_ch = self.read_ch_tx.lock().await;
+            reach_ch.take();
+        }
+        {
+            let obs = self.obs.lock().await;
+            obs.on_closed(self.loc_addr).await;
+        }
+
+        Ok(())
+    }
 }
